/**
 ** rfidRunTaskCodels.cc
 **
 ** Codels called by execution task rfidRunTask
 **
 ** Author: Thierry GERMA
 ** Date: 2008-08-26
 **
 **/ 

#include <portLib.h>

#include "server/rfidHeader.h"

#include "highgui.h"
#include "cv.h"

#include "includes_rfid.h"
#define NBANT 8
#define ANGLERES 10
#define ANGLE_DISC 10
#define DIST_DISC 25

//-----------------------------------------------------------------------------
//		Local variables
int nbTags = 0;
RFID_TAG* Tags = NULL;
static RFID_TAGLIST_POSTER_STR* posterRFID = NULL;

//angle horizontal d ouverture de la camera
static POSTER_ID platinePosterId;
static PLATINE_STATES *platinePoster;
int platineOn = 0;
double theta = 45.0;

//Neceassaire a l affichage de la carte
CvPoint zone_vis1[2+ANGLERES];
CvPoint* curveArr[1];
int curvepts[1];
IplImage * rfid;
IplImage* rfidAff;

//-----------------------------------------------------------------------------
//		Local functions

void acquire();




/*------------------------------------------------------------------------
 *
 * rfidRunTaskInit  --  Initialization codel (fIDS, ...)
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
rfidRunTaskInit(int *report)
{
	rfid = cvCreateImage(cvSize(500,500),8,3);
	rfidAff = cvCreateImage(cvSize(300,300),8,3);
	
	strcpy(SDI_F->driverParams.deviceName, RFID_PORT_NAME);
	
	
	SDI_F->pomTag = RFID_DEF_TAGGING;
	
	SDI_F->display.display = 0;
	SDI_F->display.size = 300;
	SDI_F->display.posx = 0;
	SDI_F->display.posy = 0;
	
	/* Get the poster address */
	printf("RFID_TAGLIST_POSTER_ID : %d\n", RFID_TAGLIST_POSTER_ID);
	posterRFID = (RFID_TAGLIST_POSTER_STR*) posterAddr(RFID_TAGLIST_POSTER_ID);
	if (posterRFID == NULL) 
	{
		*report = errnoGet();
		return ERROR;
	}
	posterTake(RFID_TAGLIST_POSTER_ID, POSTER_WRITE);
	posterRFID->list.nbTags = nbTags;
	for(int i=0;i<RFID_NB_MAX_TAG;i++)
	{
		sprintf(posterRFID->list.tags[i].tagId, "\0");
		posterRFID->list.tags[i].Tag_MEMORY = NULL;
		posterRFID->list.tags[i].angleMean = 0.0;
		posterRFID->list.tags[i].angleCov = 0.0;
		posterRFID->list.tags[i].distMean = 0.0;
		posterRFID->list.tags[i].distCov = 0.0;
		for(int j=0;j<8;j++)
		{
			posterRFID->list.tags[i].antennas[j] = 0;
		}
	}
	posterGive(RFID_TAGLIST_POSTER_ID);
  return OK;
}

/*------------------------------------------------------------------------
 *
 * rfidRunTaskEnd  --  Termination codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
rfidRunTaskEnd(void)
{
	cvReleaseImage( &rfid );
	cvReleaseImage( &rfidAff );
	
	
	CAENRFIDErrorCodes error=close_RFID();
    if(error!=CAENRFID_StatusOK)
		printf("Fermeture raté\n\n\n");
	
  return OK;
}

/*------------------------------------------------------------------------
 *
 * rfidRunTaskExec  --  codel of permanent activity
 *
 * Description: 
 * 
 * Reports:      OK
 * 
 * Returns:    OK or ERROR
 */

STATUS
rfidRunTaskExec(int *report)
{		

	
	if(SDI_F->display.display != 0)
	{
		cvZero(rfid);
			//Affichage des detection RFID
		for(int dist=450;dist>DIST_DISC/2;dist-=DIST_DISC)
		{
			for(int angle=0;angle<360;angle+=ANGLE_DISC)
			{
				int k=0;
				zone_vis1[k++] = cvPoint(250,250);
				zone_vis1[k++] = cvPoint(250 + (int)(dist*cos((angle+90-ANGLE_DISC/2)*M_PI/180.0))/2, 250 + (int)(dist*sin((angle+90-ANGLE_DISC/2)*M_PI/180.0))/2);
				zone_vis1[k++] = cvPoint(250 + (int)(dist*cos((angle+90+ANGLE_DISC/2)*M_PI/180.0))/2, 250 + (int)(dist*sin((angle+90+ANGLE_DISC/2)*M_PI/180.0))/2);
				
				zone_vis1[k++] = cvPoint(250, 250);
				curveArr[0] = zone_vis1; curvepts[0] = k;
				double proba = 0.0;
				for(int i=0;i<nbTags;i++)
				{
					double d = pow((dist - Tags[i].distMean)/Tags[i].distCov,2);
					double a = pow((angle - Tags[i].angleMean)/Tags[i].angleCov,2);
					proba += exp(-0.5*(d+a));
				}
				CvScalar color;
				if(Tags!=NULL)
				{
					color = CV_RGB((int)(255*proba),(int)(255*proba),(int)(255*proba));
				}
				else
				{
					color = CV_RGB((int)(255*proba),0,0);
				}
				cvFillPoly(rfid,curveArr,curvepts, 1,color);
			}
	    }
				
				//Affichage du robot + zone de detection
		cvCircle(rfid, cvPoint(250,250),19, CV_RGB(255,0,0), CV_FILLED);
		cvCircle(rfid, cvPoint(250,250),225, CV_RGB(255,255,255), 1);
		
				//Affichage du champ de vue de la camera
		if(platineOn)
		{
			if (posterTake(platinePosterId, POSTER_READ) == OK) 
			{
				if(platinePoster != NULL)
				{
					double alphaCam = -platinePoster->stateDeg.pos.pan;
					posterGive(platinePosterId);
					cvEllipse( rfid, cvPoint(250, 250), cvSize(225, 225), -90.0, -alphaCam-theta/2.0, -alphaCam+theta/2.0, CV_RGB(255, 0, 0),4);
				}
			}
		}

		cvResize(rfid,rfidAff,CV_INTER_LINEAR);
		cvShowImage("RFIDRadar", rfidAff);
		cvWaitKey(2);
	}
	
  return OK;
}
/*------------------------------------------------------------------------
 * InitDriver
 *
 * Description: 
 *
 * Reports:      OK
 *              S_rfid_INIT_FAILED
 */

/* rfidInitDriver  -  codel EXEC of InitDriver
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rfidInitDriver(RFID_DRIVER_PARAMS *driverParams, int *report)
{
	printf("initializeRFID Debut\n");
	CAENRFIDErrorCodes error = init_RFID(driverParams->deviceName);
	printf("initializeRFID termine\n");
    if(error!=CAENRFID_StatusOK)
		printf("Initialisation raté\n\n\n");
	
  /* ... add your code here ... */
  return ETHER;
}

/*------------------------------------------------------------------------
 * PomTagging
 *
 * Description: 
 *
 * Reports:      OK
 */

/* rfidPomTaggingExec  -  codel EXEC of PomTagging
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rfidPomTaggingExec(RFID_POM_TAGGING *pomTag, int *report)
{
	printf("rfidPomTaggingExec : Not yet done !!!\n");
	/*int i;

   	SDI_F->pomTag = *pomTag;

   // (re)connect to pom 
   for(i=0;i<SDI_F->maxBanks; i++)
      if (cameraPomOpen(&bankP[i], bilan) != OK) {
	SDI_F->tagP = CAMERA_DISABLE_TAGS;
	return ETHER;
      }*/

  return ETHER;
}

/*------------------------------------------------------------------------
 * EnableDisplay
 *
 * Description: 
 *
 * Reports:      OK
 */

/* rfidEnableDisplayExec  -  codel EXEC of EnableDisplay
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rfidEnableDisplayExec(RFID_DISPLAY *display, int *report)
{
	SDI_F->display.display = display->display;
	platineOn = 0;
	platinePoster = NULL;
	if(SDI_F->display.display == 0)
	{
		cvDestroyWindow("RFIDRadar");
	}
	else
	{
		cvNamedWindow("RFIDRadar", 1);
		cvMoveWindow( "RFIDRadar", display->posx, display->posy );
		
		char platinePosterName[256];
		sprintf(platinePosterName, "platineState");
		if(posterFind(platinePosterName, (POSTER_ID *)&platinePosterId) == ERROR)
		{
			h2perror("platine posterFind");
			*report = errnoGet();
			return ETHER;
		}
		if (posterTake(platinePosterId, POSTER_READ) != OK) 
		{
			h2perror("platine posterTake");
			*report = errnoGet();
			return ETHER;
		}
		platinePoster = (PLATINE_STATES *)posterAddr(platinePosterId);
		if (platinePoster == NULL) 
		{ 
			h2perror("no platine poster");
			*report = errnoGet();
			return ETHER;
		}
		posterGive(platinePosterId);
		platineOn = 1;
	}
	return ETHER;
}

/*------------------------------------------------------------------------
 * OneShot
 *
 * Description: 
 *
 * Reports:      OK
 *              S_rfid_NOT_INITIALIZED
 */

/* rfidOneShotExec  -  codel EXEC of OneShot
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rfidOneShotExec(int *report)
{
	acquire();
	return ETHER;
}

/*------------------------------------------------------------------------
 * Freeze
 *
 * Description: 
 *
 * Reports:      OK
 *              S_rfid_ACQ_FREEZE
 */

/* rfidFreezeExec  -  codel EXEC of Freeze
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rfidFreezeExec(int *report)
{
  return ETHER;
}

/*------------------------------------------------------------------------
 * Acq
 *
 * Description: 
 *
 * Reports:      OK
 *              S_rfid_NOT_INITIALIZED
 */

/* rfidAcquireStart  -  codel START of Acq
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rfidAcquireStart(int *report)
{
  return EXEC;
}

/* rfidAcquireExec  -  codel EXEC of Acq
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rfidAcquireExec(int *report)
{
	acquire();
	return EXEC;
}

/* rfidAcquireEnd  -  codel END of Acq
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rfidAcquireEnd(int *report)
{
  return ETHER;
}

/* rfidAcquireInter  -  codel INTER of Acq
   Returns:  INTER ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rfidAcquireInter(int *report)
{
  return ETHER;
}


//-----------------------------------------------------------------------------
//		Local functions

void acquire()
{
	externalTag *ordA = NULL;
	internalTag *ord = NULL;
	nbTags = acquisition();
	/* Update the poster */
	posterTake(RFID_TAGLIST_POSTER_ID, POSTER_WRITE);
	posterRFID->list.nbTags = nbTags;
	if(nbTags !=0)
	{
		if(Tags != NULL)
		{
			delete [] Tags;
		}
		
		Tags = new RFID_TAG[nbTags];
		
		printf("Detected tags : %d\n", nbTags);
		for(int i=0;i<nbTags;i++)
		{
			ord = get_Internal_Tag_by_pos(i);
			ordA = getTagInfo(ord);
			ordA->Tag_MEMORY = NULL;
			sprintf(Tags[i].tagId, "%s", ordA->Tag_ID);
			Tags[i].Tag_MEMORY = NULL;
			Tags[i].angleMean = ordA->angle;
			Tags[i].angleCov = ordA->ecartTypeAngle;
			Tags[i].distMean = ordA->longueur;
			Tags[i].distCov = ordA->ecartTypeLongueur;
			printf("%d Tag[%s] : {", i, Tags[i].tagId);
			//for(int j=0;j<8;j++)
			//{
			//	printf("%d,", ord->last_detec[j]);
			//}
			printf("(%lf, %lf, %lf, %lf)\n",Tags[i].angleMean, Tags[i].angleCov, Tags[i].distMean, Tags[i].distCov);
			delete ordA;
			
			
			/* Update the poster */
			sprintf(posterRFID->list.tags[i].tagId, "%s", Tags[i].tagId);
			posterRFID->list.tags[i].Tag_MEMORY = NULL;
			posterRFID->list.tags[i].angleMean = Tags[i].angleMean;
			posterRFID->list.tags[i].angleCov = Tags[i].angleCov;
			posterRFID->list.tags[i].distMean = Tags[i].distMean;
			posterRFID->list.tags[i].distCov = Tags[i].distCov;
			for(int j=0;j<8;j++)
			{
				posterRFID->list.tags[i].antennas[j] = ord->last_detec[j];
			}
		}
	}
	posterGive(RFID_TAGLIST_POSTER_ID);
}
